unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, GLU, GL, ExtCtrls, math;

type
  TPixelArray  = array [0..0] of Byte;
  PPixelArray  = ^TPixelArray;

  TForm1 = class(TForm)
    Timer1: TTimer;
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
    DC:HDC;
    hrc:HGLRC;
    TextureW,TextureH:Integer;
    Texture:PPixelArray;
    quadObj:PGLUquadricObj;
    TX,TY:Real;
    RX,RY,RZ:Integer;
    SX,SY:real;
    procedure PixelFormat;
    procedure GetTexture;
    function RoundUpToPowerOf2(Value: Integer):Integer;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormPaint(Sender: TObject);
var
 ps:TPaintStruct;
begin
 BeginPaint(Handle,ps);
 glClearColor(0,0,0,1);

 glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

 glEnable(GL_DEPTH_TEST);

 glPushMatrix;
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
 glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_DECAL);
 glTexImage2d(GL_TEXTURE_2D,0,3,TextureW,TextureH,0,GL_RGB,GL_UNSIGNED_BYTE,Texture^);
 glEnable(GL_TEXTURE_GEN_S);
 glEnable(GL_TEXTURE_GEN_T);
 glEnable(GL_TEXTURE_2D);
 glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
 gluQuadricTexture(quadObj,GL_TRUE);

 glTranslatef(TX,TY,0);
 glRotatef(RX,1,0,0);
 glRotatef(RY,0,1,0);
 glRotatef(RZ,0,0,1);
 gluSphere(quadObj,2.7, 15, 10);

 TX:=TX+SX;
 TY:=TY+SY;
 if (TX<-10) or (TX>10) then
  SX:=-SX;
 if (TY<-10) or (TY>10) then
  SY:=-SY;

 glDisable(GL_TEXTURE_2D);
 glPopMatrix;
 glDisable(GL_DEPTH_TEST);

 glFlush();
 swapBuffers(dc);
 EndPaint(Handle,ps);
end;

procedure TForm1.FormResize(Sender: TObject);
begin
 glMatrixMode(GL_PROJECTION);
 glLoadIdentity( );

 if width <= height then
  glOrtho (-10, 10, -10*height/width, 10*height/width, -10.0, 10.0)
 else
  glOrtho (-10*width/height, 10*width/height, -10, 10, -10.0, 10.0);

 glViewport(0, 0, width, height);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 wglMakeCurrent(0,0);
 wglDeleteContext(hrc);
 ReleaseDC(handle,DC);
end;

procedure TForm1.PixelFormat;
var
 nPixelFormat:Integer;
 pfd:TPixelFormatDescriptor;
begin
 FillChar(pfd,Sizeof(pfd),0);

 pfd.nSize:=sizeof(pfd);
 pfd.nVersion:=1;
 pfd.dwFlags:=PFD_DOUBLEBUFFER+PFD_SWAP_LAYER_BUFFERS+PFD_SUPPORT_OPENGL+PFD_DRAW_TO_WINDOW+PFD_GENERIC_ACCELERATED;
 pfd.iPixelType:=PFD_TYPE_RGBA;
 pfd.cColorBits:=24;
 pfd.cAlphaBits:=64;
 pfd.cAccumBits:=64;
 pfd.cDepthBits:=32;
 pfd.cStencilBits:=64;
 pfd.iLayerType:=PFD_MAIN_PLANE;

 nPixelFormat:=ChoosePixelFormat(DC,@pfd);
 if nPixelFormat<>0 then
  SetPixelFormat(DC,nPixelFormat,@pfd);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 DC:=GetDC(Handle);
 PixelFormat;
 hrc:=wglCreateContext(DC);
 wglMakeCurrent(DC,hrc);

 glColor3f(1,0,0);

 RX:=0;RY:=0;RZ:=0;
 quadObj:=gluNewQuadric();
 GetTexture;

 randomize;
 TX:=random(10)/4;
 TY:=random(10)/5;
 SX:=random(10)/100;
 if SX>5 then
  SX:=-SX;
 SY:=random(10)/100;
 if SY<5 then
  SY:=-SY;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
 RX:=RX+3;
 RZ:=RZ+3;
 if RX>360 then RX:=RX-360;
 if RZ>360 then RZ:=RZ-360;
 FormPaint(nil);
end;

procedure TForm1.GetTexture;
var
 BMInfo      :TBitmapInfo;
 I,ImageSize :Integer;
 Temp        :Byte;
 Bitmap      :Graphics.TBitmap;
 TempImage   :TBitmap;
 MemDC       : HDC;
begin
 Bitmap:=TBitmap.Create;
 TempImage:=TBitmap.Create;
 TempImage.Width:=32;
 TempImage.Height:=32;
 TempImage.Canvas.Draw(0,0,Icon);
 Bitmap.Width:=RoundUpToPowerOf2(TempImage.Width);
 Bitmap.Height:=RoundUpToPowerOf2(TempImage.Height);
 Bitmap.Canvas.CopyRect(Rect(0,0,Bitmap.Width,Bitmap.Height), TempImage.Canvas,
                        Rect(0,0,32,32));
 MemDC:=CreateCompatibleDC(0);
 try
  with bitmap do
   begin
    ImageSize:=bitmap.Width*bitmap.Height;
    TextureW:=bitmap.Width;
    TextureH:=bitmap.Height;

    GetMem(Texture,ImageSize*3);
    MemDC:=CreateCompatibleDC(0);
    FillChar(BMInfo,SizeOf(BMInfo),0);
    With BMInfo.bmiHeader do
     begin
      biSize:=sizeof(TBitmapInfoHeader);
      biBitCount:=24;
      biWidth:=bitmap.Width;
      biHeight:=-bitmap.Height;
      biPlanes:=1;
      biCompression:=BI_RGB;
     end;
    GetDIBits(MemDC,Bitmap.Handle,0,bitmap.Height,Texture,BMInfo,DIB_RGB_COLORS);
  {$R-}
    for i:=0 to ImageSize-1 do
     begin
      Temp:=Texture^[i*3];
      Texture^[I*3]:=Texture^[I*3+2];
      Texture^[I*3+2]:=Temp;
     end;
  {$R-}
    glPixelStorei(GL_UNPACK_ALIGNMENT,0);
    glPixelStorei(GL_UNPACK_ROW_LENGTH,0);
    glPixelStorei(GL_UNPACK_SKIP_ROWS,0);
    glPixelStorei(GL_UNPACK_SKIP_PIXELS,0);
   end;
 finally
  Bitmap.Free;
  TempImage.Free;
  DeleteDC(MemDC);
 end;
end;

function TForm1.RoundUpToPowerOf2(Value: Integer): Integer;
var
  LogTwo : Extended;
begin
 LogTwo:=log2(Value);
 if Trunc(LogTwo) < LogTwo then
  Result:=Trunc(Power(2,Trunc(LogTwo)+1))
 else
  Result:=value;
end;

end.

